#include "asynctmr.h"
#include <ansi_c.h>
#include <utility.h>
#include <formatio.h>
#include <rs232.h>
#include <cvirte.h>		
#include <userint.h>
#include "adam.h"

static int panelHandle;
char send_data[20],read_data[20];
double temperatura;
char *garbage;
int minimum, maksimum;
int functionId;
int minimum_stari=20;
int maksimum_stari=30;
int lock;
int promena;


int CVICALLBACK DrugiOtkucaj (int reserved, int timerId, int event, void *callbackData, int eventData1, int eventData2);


int main (int argc, char *argv[])
   {
   if (InitCVIRTE (0, argv, 0) == 0)
      return -1;	/* out of memory */
   if ((panelHandle = LoadPanel (0, "adam.uir", PANEL)) < 0)
      return -1;
   DisplayPanel (panelHandle);
   
   CmtNewLock (NULL, 0, &lock); //definisanje locka za multithreading
	
   //otvaranja serijskog porta
   OpenComConfig (1, "COM1", 9600, 0, 8, 1, 512, 512);//COM5, 9600b/s, bez parnosti, 1 stop bit
	
   //definisanje da se na displeju stampa sa PC-a, format je $AA82
   strcpy(send_data,"$0282\r");
   ComWrt (1, send_data, 6); //sa portCOM5 se iz niza send_data salje 6 bajtova
   Delay(0.5);//cekam da stigne odziv za adrese 02, da ne bi doslo do zagusenja POLUDUPLEX magistrale
   //rad jednostavnosti samo cekanje, alternativa da se odziv cita i tumaci, sto bi trajalo krace ali 
   //se komplikuje program
	
   //definisanje vrednosti donjeg praga na analognom modulu, koji se koristi lokalno
   strcpy(send_data,"@03LO+020.00\r");
   ComWrt (1, send_data, 13);
   Delay(0.5);
	
   //definisanje vrednosti gornjeg praga na analognom modulu, koji se koristi lokalno
   strcpy(send_data,"@03HI+030.00\r");
   ComWrt (1, send_data, 13);
   Delay(0.5);
	
   //definisanje tipa alarma na analognom modulu  - tenutni, tj. ne lecuje se
   strcpy(send_data,"@03EAM\r");
   ComWrt (1, send_data, 7);
   Delay(0.5);
	
    
   NewAsyncTimer (0.5, -1, 1, DrugiOtkucaj, 0);    //definisanje asinhronog tajmera, u cijem "interaptu"
   //se izvrsava drugi thread, funkcija DrugiOtkucaj
   
   RunUserInterface ();
   CloseCom(1);//zatvaranje porta
	
   DiscardPanel (panelHandle);
	
   CmtDiscardLock (lock);//dealokacija locka za multithreading, oslobadjanje memorijskih resursa
	
	
   return 0;
   }


int CVICALLBACK quit (int panel, int event, void *callbackData,int eventData1, int eventData2)
   {
   switch (event)
      {
	  case EVENT_CLOSE:
	     QuitUserInterface (0);
		 break;
	  }
   return 0;
   }

// glavni thread, funkcija koja se izvrsava u "prekidu" glavnog tajmera (onog u UIR fajlu)
//ovde se izvrsava komunikacija na RS485 magistrali
int CVICALLBACK otkucaj (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   int promena_temp;
   char string[3];
   switch (event)
      {
	  case EVENT_TIMER_TICK:
		 //brisem sve dosadasnje odzive kao nebitne, radi jednostavnosti programa
		 FlushInQ(1);
		 
		 CmtGetLock(lock); //zabrana prebacivanja u drugi thread, da se ne promeni promenljiva promena
		 promena_temp=promena;
		 CmtReleaseLock(lock);//dozvola prebcivanja u drugi thread
		 
		 switch (promena_temp)
		    {
			case 0:
			   //ocitavanje analognog ulazam format #AA
		       strcpy(send_data,"#03\r");
               ComWrt (1, send_data, 4);
			   
               //odziv ima format >+0xy.zw<cr> //
			   ComRdTerm (1, read_data, 9,'\r');//sa porta 5 se u niz read_data cita 9 bajtova, sa ocekivanjem da
			   //9. bajt bude <CR>
			   
		       //prebacivanje niza read_data od cifre x u broj u floating point predstavi
			   temperatura = strtod (&read_data[3],&garbage);
		       
			   //prikaz na UI
			   SetCtrlVal (panelHandle, PANEL_TRENUTNA, temperatura);
			   
			   //slanje za prikaz na displeju, format $AA029xy.zw <cr>
			   strcpy(send_data,"$029");
               send_data[4]=read_data[3];send_data[5]=read_data[4];
		       send_data[6]='.';
		       send_data[7]=read_data[6];
               send_data[8]=read_data[7];
		       send_data[9]=' ';
               send_data[10]='\r';		  
		       ComWrt (1, send_data, 11);
		 	   //setovanje digitalnog izlaza
		 	   CmtGetLock(lock);//zabrana prebacivanja u drugi thread, da se ne promeni promenljiva minimum
		    
			   //ukljucivanje LE diode za donji prag na Digital IO modulu, format #AA1XY
			   //X je broj kanala, ovde je 7, Y moze biti 0 ili 1
			   //takodje i kontrola LE diode u UI
			   if (temperatura<minimum) 
		          {
			      strcpy(send_data,"#011701\r");
			      SetCtrlVal(panelHandle,PANEL_HLADNO,1);
		          }
               else 
		          {
			      strcpy(send_data,"#011700\r");
			      SetCtrlVal(panelHandle,PANEL_HLADNO,0);
		          }
		       CmtReleaseLock(lock); //dozvola prebacivanja u drugi thread
			   ComWrt (1, send_data, 8);
			   
			   
		       CmtGetLock(lock);//zabrana prebacivanja u drugi thread, da se ne promeni promenljiva maximum  
			   
			   //kontrola LE diode u UI
			   if (temperatura>maksimum)
		          {
			      SetCtrlVal(panelHandle,PANEL_TOPLO,1);
		          }
               else 
		          {
			      SetCtrlVal(panelHandle,PANEL_TOPLO,0);
		          }
		       CmtReleaseLock(lock);//dozvola prebacivanja u drugi thread 
		       ComWrt (1, send_data, 8);
			   break;
			case 1://promena minimuma
			   //donji alarm za analog input module, format @AALO+0XY.00<cr>
	           CmtGetLock(lock);//zabrana prebacivanja u drugi thread, da se ne promene promenljive minimum i promena
			   promena=0;
			   sprintf(string,"%d",minimum);
			   CmtReleaseLock(lock);//dozvola prebacivanja u drugi thread
			   strcpy(send_data,"@03LO+0");
			   strcpy(&send_data[7],string);
			   strcpy(&send_data[9],".00\r");
			   ComWrt (1, send_data, 13);
			   break;
			   
			case 2: //promena maximuma
			   //gornji alarm za analog input module, format @AAHI+0XY.00<cr>
			   CmtGetLock(lock); //zabrana prebacivanja u drugi thread, da se ne promene promenljive maksimum i promena
			   promena=0;
			   sprintf(string,"%d",maksimum);
			   CmtReleaseLock(lock);//dozvola prebacivanja u drugi thread
			   strcpy(send_data,"@03HI+0");
			   strcpy(&send_data[7],string);
			   strcpy(&send_data[9],".00\r");
			   ComWrt (1, send_data, 13);
			   break;
		 }
		 break;
      }
   return 0;
   }


//drugi thread, u kome se ocitava situacija na UI, i ukoliko je doslo do promene kontrola minimum i maximum 
//pomocu promenljive promena se glavnom threadu signalizira da umesto standardnog ciklusa na magistrali
//treba da posalje analognom modulu novu vrednost gornjeg ili donjeg praga 
int CVICALLBACK DrugiOtkucaj (int reserved, int timerId, int event, void *callbackData, int eventData1, int eventData2)
   {
   int promena_temp;
   
   CmtGetLock(lock);
   promena_temp=promena;
   CmtReleaseLock(lock);
   
   if (promena_temp==0)	   
      {
	  CmtGetLock(lock);	   
      GetCtrlVal (panelHandle, PANEL_MINIMUM, &minimum);
      CmtReleaseLock(lock); 
      CmtGetLock(lock);	   
      if (minimum!=minimum_stari)
         {
         promena=1;
	     minimum_stari=minimum;
	     }
      CmtReleaseLock(lock);
   
      CmtGetLock(lock);
      GetCtrlVal (panelHandle, PANEL_MAKSIMUM, &maksimum);
      CmtReleaseLock(lock);
      
	  CmtGetLock(lock);	   
      if (maksimum!=maksimum_stari)
         {
         promena=2;
	     maksimum_stari=maksimum;	  
	     }
      CmtReleaseLock(lock);
      }
   return 0;
   }

